package javax.swing;

import javax.accessibility.AccessibleRole;
import java.awt.event.ComponentEvent;
import java.io.Serializable;
import java.awt.event.ComponentAdapter;
import javax.accessibility.AccessibleContext;
import java.awt.Composite;
import java.awt.AlphaComposite;
import java.awt.Graphics2D;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.peer.ComponentPeer;
import javax.swing.event.ChangeListener;
import java.awt.LayoutManager;
import java.beans.Transient;
import java.awt.Graphics;
import java.awt.Insets;
import javax.swing.border.Border;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Rectangle;
import java.awt.Component;
import javax.swing.plaf.ViewportUI;
import javax.swing.event.ChangeEvent;
import java.awt.event.ComponentListener;
import java.awt.Image;
import java.awt.Point;
import javax.accessibility.Accessible;

public class JViewport extends JComponent implements Accessible
{
    private static final String uiClassID = "ViewportUI";
    static final Object EnableWindowBlit;
    protected boolean isViewSizeSet;
    protected Point lastPaintPosition;
    @Deprecated
    protected boolean backingStore;
    protected transient Image backingStoreImage;
    protected boolean scrollUnderway;
    private ComponentListener viewListener;
    private transient ChangeEvent changeEvent;
    public static final int BLIT_SCROLL_MODE = 1;
    public static final int BACKINGSTORE_SCROLL_MODE = 2;
    public static final int SIMPLE_SCROLL_MODE = 0;
    private int scrollMode;
    private transient boolean repaintAll;
    private transient boolean waitingForRepaint;
    private transient Timer repaintTimer;
    private transient boolean inBlitPaint;
    private boolean hasHadValidView;
    private boolean viewChanged;
    
    public JViewport() {
        this.isViewSizeSet = false;
        this.lastPaintPosition = null;
        this.backingStore = false;
        this.backingStoreImage = null;
        this.scrollUnderway = false;
        this.viewListener = null;
        this.changeEvent = null;
        this.scrollMode = 1;
        this.setLayout(this.createLayoutManager());
        this.setOpaque(true);
        this.updateUI();
        this.setInheritsPopupMenu(true);
    }
    
    public ViewportUI getUI() {
        return (ViewportUI)this.ui;
    }
    
    public void setUI(ViewportUI ui) {
        super.setUI(ui);
    }
    
    @Override
    public void updateUI() {
        this.setUI((ViewportUI)UIManager.getUI(this));
    }
    
    @Override
    public String getUIClassID() {
        return "ViewportUI";
    }
    
    @Override
    protected void addImpl(Component view, Object o, int n) {
        this.setView(view);
    }
    
    @Override
    public void remove(Component comp) {
        comp.removeComponentListener(this.viewListener);
        super.remove(comp);
    }
    
    @Override
    public void scrollRectToVisible(Rectangle rectangle) {
        Component view = this.getView();
        if (view == null) {
            return;
        }
        if (!view.isValid()) {
            this.validateView();
        }
        int positionAdjustment = this.positionAdjustment(this.getWidth(), rectangle.width, rectangle.x);
        int positionAdjustment2 = this.positionAdjustment(this.getHeight(), rectangle.height, rectangle.y);
        if (positionAdjustment != 0 || positionAdjustment2 != 0) {
            Point viewPosition = this.getViewPosition();
            Dimension size = view.getSize();
            int x = viewPosition.x;
            int y = viewPosition.y;
            Dimension extentSize = this.getExtentSize();
            viewPosition.x -= positionAdjustment;
            viewPosition.y -= positionAdjustment2;
            if (view.isValid()) {
                if (this.getParent().getComponentOrientation().isLeftToRight()) {
                    if (viewPosition.x + extentSize.width > size.width) {
                        viewPosition.x = Math.max(0, size.width - extentSize.width);
                    }
                    else if (viewPosition.x < 0) {
                        viewPosition.x = 0;
                    }
                }
                else if (extentSize.width > size.width) {
                    viewPosition.x = size.width - extentSize.width;
                }
                else {
                    viewPosition.x = Math.max(0, Math.min(size.width - extentSize.width, viewPosition.x));
                }
                if (viewPosition.y + extentSize.height > size.height) {
                    viewPosition.y = Math.max(0, size.height - extentSize.height);
                }
                else if (viewPosition.y < 0) {
                    viewPosition.y = 0;
                }
            }
            if (viewPosition.x != x || viewPosition.y != y) {
                this.setViewPosition(viewPosition);
                this.scrollUnderway = false;
            }
        }
        return;
    }
    
    private void validateView() {
        Container validateRoot = SwingUtilities.getValidateRoot(this, false);
        if (validateRoot == null) {
            return;
        }
        validateRoot.validate();
        RepaintManager currentManager = RepaintManager.currentManager(this);
        if (currentManager != null) {
            currentManager.removeInvalidComponent((JComponent)validateRoot);
        }
        return;
    }
    
    private int positionAdjustment(int n, int n2, int n3) {
        if (n3 >= 0 && n2 + n3 <= n) {
            return 0;
        }
        if (n3 <= 0 && n2 + n3 >= n) {
            return 0;
        }
        if (n3 > 0 && n2 <= n) {
            return -n3 + n - n2;
        }
        if (n3 >= 0 && n2 >= n) {
            return -n3;
        }
        if (n3 <= 0 && n2 <= n) {
            return -n3;
        }
        if (n3 < 0 && n2 >= n) {
            return -n3 + n - n2;
        }
        return 0;
    }
    
    @Override
    public final void setBorder(Border border) {
        if (border != null) {
            throw new IllegalArgumentException("JViewport.setBorder() not supported");
        }
        return;
    }
    
    @Override
    public final Insets getInsets() {
        return new Insets(0, 0, 0, 0);
    }
    
    @Override
    public final Insets getInsets(Insets insets) {
        int n = 0;
        insets.bottom = n;
        insets.right = n;
        insets.top = n;
        insets.left = n;
        return insets;
    }
    
    private Graphics getBackingStoreGraphics(Graphics graphics) {
        Graphics graphics2 = this.backingStoreImage.getGraphics();
        graphics2.setColor(graphics.getColor());
        graphics2.setFont(graphics.getFont());
        graphics2.setClip(graphics.getClipBounds());
        return graphics2;
    }
    
    private void paintViaBackingStore(Graphics graphics) {
        Graphics backingStoreGraphics = this.getBackingStoreGraphics(graphics);
        try {
            super.paint(backingStoreGraphics);
            graphics.drawImage(this.backingStoreImage, 0, 0, this);
        }
        finally {
            backingStoreGraphics.dispose();
        }
    }
    
    private void paintViaBackingStore(Graphics graphics, Rectangle clip) {
        Graphics backingStoreGraphics = this.getBackingStoreGraphics(graphics);
        try {
            super.paint(backingStoreGraphics);
            graphics.setClip(clip);
            graphics.drawImage(this.backingStoreImage, 0, 0, this);
        }
        finally {
            backingStoreGraphics.dispose();
        }
    }
    
    @Override
    public boolean isOptimizedDrawingEnabled() {
        return false;
    }
    
    @Override
    protected boolean isPaintingOrigin() {
        return this.scrollMode == 2;
    }
    
    private Point getViewLocation() {
        Component view = this.getView();
        if (view != null) {
            return view.getLocation();
        }
        return new Point(0, 0);
    }
    
    @Override
    public void paint(Graphics graphics) {
        int width = this.getWidth();
        int height = this.getHeight();
        if (width <= 0 || height <= 0) {
            return;
        }
        if (this.inBlitPaint) {
            super.paint(graphics);
            return;
        }
        if (this.repaintAll) {
            this.repaintAll = false;
            Rectangle clipBounds2 = graphics.getClipBounds();
            if (clipBounds2.width < this.getWidth() || clipBounds2.height < this.getHeight()) {
                this.waitingForRepaint = true;
                if (this.repaintTimer == null) {
                    this.repaintTimer = this.createRepaintTimer();
                }
                this.repaintTimer.stop();
                this.repaintTimer.start();
            }
            else {
                if (this.repaintTimer != null) {
                    this.repaintTimer.stop();
                }
                this.waitingForRepaint = false;
            }
        }
        else if (this.waitingForRepaint) {
            Rectangle clipBounds = graphics.getClipBounds();
            if (clipBounds.width >= this.getWidth() && clipBounds.height >= this.getHeight()) {
                this.waitingForRepaint = false;
                this.repaintTimer.stop();
            }
        }
        if (!this.backingStore || this.isBlitting() || this.getView() == null) {
            super.paint(graphics);
            this.lastPaintPosition = this.getViewLocation();
            return;
        }
        Rectangle bounds = this.getView().getBounds();
        if (!this.isOpaque()) {
            graphics.clipRect(0, 0, bounds.width, bounds.height);
        }
        if (this.backingStoreImage == null) {
            this.backingStoreImage = this.createImage(width, height);
            Rectangle clipBounds4 = graphics.getClipBounds();
            if (clipBounds4.width != width || clipBounds4.height != height) {
                if (!this.isOpaque()) {
                    graphics.setClip(0, 0, Math.min(bounds.width, width), Math.min(bounds.height, height));
                }
                else {
                    graphics.setClip(0, 0, width, height);
                }
                this.paintViaBackingStore(graphics, clipBounds4);
            }
            else {
                this.paintViaBackingStore(graphics);
            }
        }
        else if (!this.scrollUnderway || this.lastPaintPosition.equals(this.getViewLocation())) {
            this.paintViaBackingStore(graphics);
        }
        else {
            Point point = new Point();
            Point point2 = new Point();
            Dimension dimension = new Dimension();
            Rectangle r = new Rectangle();
            Point viewLocation = this.getViewLocation();
            if (!this.computeBlit(viewLocation.x - this.lastPaintPosition.x, viewLocation.y - this.lastPaintPosition.y, point, point2, dimension, r)) {
                this.paintViaBackingStore(graphics);
            }
            else {
                int n = point2.x - point.x;
                int n2 = point2.y - point.y;
                Rectangle clipBounds3 = graphics.getClipBounds();
                graphics.setClip(0, 0, width, height);
                Graphics backingStoreGraphics = this.getBackingStoreGraphics(graphics);
                try {
                    backingStoreGraphics.copyArea(point.x, point.y, dimension.width, dimension.height, n, n2);
                    graphics.setClip(clipBounds3.x, clipBounds3.y, clipBounds3.width, clipBounds3.height);
                    backingStoreGraphics.setClip(bounds.intersection(r));
                    super.paint(backingStoreGraphics);
                    graphics.drawImage(this.backingStoreImage, 0, 0, this);
                }
                finally {
                    backingStoreGraphics.dispose();
                }
            }
        }
        this.lastPaintPosition = this.getViewLocation();
        this.scrollUnderway = false;
        return;
    }
    
    @Override
    public void reshape(int n, int n2, int n3, int n4) {
        int n5 = this.getWidth() != n3 || this.getHeight() != n4;
        if (!(n5 == 0)) {
            this.backingStoreImage = null;
        }
        super.reshape(n, n2, n3, n4);
        if (n5 || this.viewChanged) {
            this.viewChanged = false;
            this.fireStateChanged();
        }
    }
    
    public void setScrollMode(int scrollMode) {
        this.scrollMode = scrollMode;
        this.backingStore = (scrollMode == 2);
    }
    
    public int getScrollMode() {
        return this.scrollMode;
    }
    
    @Deprecated
    public boolean isBackingStoreEnabled() {
        return this.scrollMode == 2;
    }
    
    @Deprecated
    public void setBackingStoreEnabled(boolean b) {
        if (b) {
            this.setScrollMode(2);
        }
        else {
            this.setScrollMode(1);
        }
    }
    
    private boolean isBlitting() {
        Component view = this.getView();
        return this.scrollMode == 1 && view instanceof JComponent && view.isOpaque();
    }
    
    public Component getView() {
        return (this.getComponentCount() > 0) ? this.getComponent(0) : null;
    }
    
    public void setView(Component comp) {
        for (int i = this.getComponentCount() - 1; i >= 0; --i) {
            this.remove(this.getComponent(i));
        }
        this.isViewSizeSet = false;
        if (comp != null) {
            super.addImpl(comp, null, -1);
            comp.addComponentListener(this.viewListener = this.createViewListener());
        }
        if (this.hasHadValidView) {
            this.fireStateChanged();
        }
        else if (comp != null) {
            this.hasHadValidView = true;
        }
        this.viewChanged = true;
        this.revalidate();
        this.repaint();
    }
    
    public Dimension getViewSize() {
        Component view = this.getView();
        if (view == null) {
            return new Dimension(0, 0);
        }
        if (this.isViewSizeSet) {
            return view.getSize();
        }
        return view.getPreferredSize();
    }
    
    public void setViewSize(Dimension size) {
        Component view = this.getView();
        if (view != null && !size.equals(view.getSize())) {
            this.scrollUnderway = false;
            view.setSize(size);
            this.isViewSizeSet = true;
            this.fireStateChanged();
        }
    }
    
    public Point getViewPosition() {
        Component view = this.getView();
        if (view != null) {
            Point location = view.getLocation();
            location.x = -location.x;
            location.y = -location.y;
            return location;
        }
        return new Point(0, 0);
    }
    
    public void setViewPosition(Point point) {
        Component view = this.getView();
        if (view == null) {
            return;
        }
        int x = point.x;
        int y = point.y;
        int n;
        int n2;
        if ((view instanceof JComponent)) {
            JComponent component = (JComponent)view;
            n = component.getX();
            n2 = component.getY();
        }
        else {
            Rectangle bounds = view.getBounds();
            n = bounds.x;
            n2 = bounds.y;
        }
        int x2 = -x;
        int y2 = -y;
        if (n != x2 || n2 != y2) {
            if (!this.waitingForRepaint && this.isBlitting() && this.canUseWindowBlitter()) {
                RepaintManager currentManager = RepaintManager.currentManager(this);
                JComponent component2 = (JComponent)view;
                Rectangle dirtyRegion = currentManager.getDirtyRegion(component2);
                if (dirtyRegion == null || !dirtyRegion.contains(component2.getVisibleRect())) {
                    currentManager.beginPaint();
                    try {
                        Graphics safelyGetGraphics = JComponent.safelyGetGraphics(this);
                        this.flushViewDirtyRegion(safelyGetGraphics, dirtyRegion);
                        view.setLocation(x2, y2);
                        Rectangle clip = new Rectangle(0, 0, this.getWidth(), Math.min(this.getHeight(), component2.getHeight()));
                        safelyGetGraphics.setClip(clip);
                        this.repaintAll = (this.windowBlitPaint(safelyGetGraphics) && this.needsRepaintAfterBlit());
                        safelyGetGraphics.dispose();
                        currentManager.notifyRepaintPerformed(this, clip.x, clip.y, clip.width, clip.height);
                        currentManager.markCompletelyClean((JComponent)this.getParent());
                        currentManager.markCompletelyClean(this);
                        currentManager.markCompletelyClean(component2);
                    }
                    finally {
                        currentManager.endPaint();
                    }
                }
                else {
                    view.setLocation(x2, y2);
                    this.repaintAll = false;
                }
            }
            else {
                this.scrollUnderway = true;
                view.setLocation(x2, y2);
                this.repaintAll = false;
            }
            this.revalidate();
            this.fireStateChanged();
        }
        return;
    }
    
    public Rectangle getViewRect() {
        return new Rectangle(this.getViewPosition(), this.getExtentSize());
    }
    
    protected boolean computeBlit(int n, int n2, Point point, Point point2, Dimension dimension, Rectangle rectangle) {
        int abs = Math.abs(n);
        int abs2 = Math.abs(n2);
        Dimension extentSize = this.getExtentSize();
        if (n == 0 && n2 != 0 && abs2 < extentSize.height) {
            if (n2 < 0) {
                point.y = -n2;
                point2.y = 0;
                rectangle.y = extentSize.height + n2;
            }
            else {
                point.y = 0;
                point2.y = n2;
                rectangle.y = 0;
            }
            int x = 0;
            point2.x = x;
            point.x = x;
            rectangle.x = x;
            dimension.width = extentSize.width;
            dimension.height = extentSize.height - abs2;
            rectangle.width = extentSize.width;
            rectangle.height = abs2;
            return true;
        }
        else if (n2 == 0 && n != 0 && abs < extentSize.width) {
            if (n < 0) {
                point.x = -n;
                point2.x = 0;
                rectangle.x = extentSize.width + n;
            }
            else {
                point.x = 0;
                point2.x = n;
                rectangle.x = 0;
            }
            int y = 0;
            point2.y = y;
            point.y = y;
            rectangle.y = y;
            dimension.width = extentSize.width - abs;
            dimension.height = extentSize.height;
            rectangle.width = abs;
            rectangle.height = extentSize.height;
            return true;
        }
        else {
            return false;
        }
    }
    
    @Transient
    public Dimension getExtentSize() {
        return this.getSize();
    }
    
    public Dimension toViewCoordinates(Dimension d) {
        return new Dimension(d);
    }
    
    public Point toViewCoordinates(Point p) {
        return new Point(p);
    }
    
    public void setExtentSize(Dimension size) {
        if (!size.equals(this.getExtentSize())) {
            this.setSize(size);
            this.fireStateChanged();
        }
    }
    
    protected JViewport.ViewListener createViewListener() {
        return new JViewport.ViewListener();
    }
    
    protected LayoutManager createLayoutManager() {
        return ViewportLayout.SHARED_INSTANCE;
    }
    
    public void addChangeListener(ChangeListener l) {
        this.listenerList.add(ChangeListener.class, l);
    }
    
    public void removeChangeListener(ChangeListener l) {
        this.listenerList.remove(ChangeListener.class, l);
    }
    
    public ChangeListener[] getChangeListeners() {
        return (ChangeListener[])this.listenerList.getListeners(ChangeListener.class);
    }
    
    protected void fireStateChanged() {
        Object[] listenerList = this.listenerList.getListenerList();
        for (int i = listenerList.length - 2; i >= 0; i -= 2) {
            if (listenerList[i] == ChangeListener.class) {
                if (this.changeEvent == null) {
                    this.changeEvent = new ChangeEvent(this);
                }
                ((ChangeListener)listenerList[i + 1]).stateChanged(this.changeEvent);
            }
        }
    }
    
    @Override
    public void repaint(long tm, int n, int n2, int width, int height) {
        Container parent = this.getParent();
        if (parent != null) {
            parent.repaint(tm, n + this.getX(), n2 + this.getY(), width, height);
        }
        else {
            super.repaint(tm, n, n2, width, height);
        }
    }
    
    @Override
    protected String paramString() {
        return super.paramString() + ",isViewSizeSet=" + (this.isViewSizeSet ? "true" : "false") + ",lastPaintPosition=" + ((this.lastPaintPosition != null) ? this.lastPaintPosition.toString() : "") + ",scrollUnderway=" + (this.scrollUnderway ? "true" : "false");
    }
    
    @Override
    protected void firePropertyChange(String propertyName, Object oldValue, Object newValue) {
        super.firePropertyChange(propertyName, oldValue, newValue);
        if (propertyName.equals(JViewport.EnableWindowBlit)) {
            if (newValue != null) {
                this.setScrollMode(1);
            }
            else {
                this.setScrollMode(0);
            }
        }
    }
    
    private boolean needsRepaintAfterBlit() {
        Container container;
        for (container = this.getParent(); container != null && container.isLightweight(); container = container.getParent()) {}
        if (container != null) {
            ComponentPeer peer = container.getPeer();
            if (peer != null && peer.canDetermineObscurity() && !peer.isObscured()) {
                return false;
            }
        }
        return true;
    }
    
    private Timer createRepaintTimer() {
        Timer timer = new Timer(300, new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent actionEvent) {
                if (JViewport.this.waitingForRepaint) {
                    JViewport.this.repaint();
                }
            }
        });
        timer.setRepeats(false);
        return timer;
    }
    
    private void flushViewDirtyRegion(Graphics graphics, Rectangle rectangle) {
        JComponent component = (JComponent)this.getView();
        if (rectangle != null && rectangle.width > 0 && rectangle.height > 0) {
            rectangle.x += component.getX();
            rectangle.y += component.getY();
            if (graphics.getClipBounds() == null) {
                graphics.setClip(0, 0, this.getWidth(), this.getHeight());
            }
            graphics.clipRect(rectangle.x, rectangle.y, rectangle.width, rectangle.height);
            Rectangle clipBounds = graphics.getClipBounds();
            if (clipBounds.width > 0 && clipBounds.height > 0) {
                this.paintView(graphics);
            }
        }
    }
    
    private boolean windowBlitPaint(Graphics graphics) {
        int width = this.getWidth();
        int height = this.getHeight();
        if (width == 0 || height == 0) {
            return false;
        }
        RepaintManager.currentManager(this);
        JComponent component = (JComponent)this.getView();
        int n;
        if (this.lastPaintPosition == null || this.lastPaintPosition.equals(this.getViewLocation())) {
            this.paintView(graphics);
            n = false;
        }
        else {
            Point point = new Point();
            Point point2 = new Point();
            Dimension dimension = new Dimension();
            Rectangle r = new Rectangle();
            Point viewLocation = this.getViewLocation();
            if (!this.computeBlit(viewLocation.x - this.lastPaintPosition.x, viewLocation.y - this.lastPaintPosition.y, point, point2, dimension, r)) {
                this.paintView(graphics);
                n = false;
            }
            else {
                Rectangle intersection = component.getBounds().intersection(r);
                intersection.x -= component.getX();
                intersection.y -= component.getY();
                this.blitDoubleBuffered(component, graphics, intersection.x, intersection.y, intersection.width, intersection.height, point.x, point.y, point2.x, point2.y, dimension.width, dimension.height);
                n = true;
            }
        }
        this.lastPaintPosition = this.getViewLocation();
        return n;
    }
    
    private void blitDoubleBuffered(JComponent component, Graphics graphics, int n, int n2, int n3, int n4, int n5, int n6, int n7, int n8, int n9, int n10) {
        RepaintManager currentManager = RepaintManager.currentManager(this);
        int n11 = n7 - n5;
        int n12 = n8 - n6;
        Composite composite = null;
        if ((graphics instanceof Graphics2D)) {
            Graphics2D graphics2D = (Graphics2D)graphics;
            composite = graphics2D.getComposite();
            graphics2D.setComposite(AlphaComposite.Src);
        }
        currentManager.copyArea(this, graphics, n5, n6, n9, n10, n11, n12, false);
        if (composite != null) {
            ((Graphics2D)graphics).setComposite(composite);
        }
        int x = component.getX();
        int y = component.getY();
        graphics.translate(x, y);
        graphics.setClip(n, n2, n3, n4);
        component.paintForceDoubleBuffered(graphics);
        graphics.translate(-x, -y);
    }
    
    private void paintView(Graphics graphics) {
        Rectangle clipBounds = graphics.getClipBounds();
        JComponent component = (JComponent)this.getView();
        if (component.getWidth() >= this.getWidth()) {
            int x = component.getX();
            int y = component.getY();
            graphics.translate(x, y);
            graphics.setClip(clipBounds.x - x, clipBounds.y - y, clipBounds.width, clipBounds.height);
            component.paintForceDoubleBuffered(graphics);
            graphics.translate(-x, -y);
            graphics.setClip(clipBounds.x, clipBounds.y, clipBounds.width, clipBounds.height);
        }
        else {
            try {
                this.inBlitPaint = true;
                this.paintForceDoubleBuffered(graphics);
            }
            finally {
                this.inBlitPaint = false;
            }
        }
    }
    
    private boolean canUseWindowBlitter() {
        if (!this.isShowing() || (!(this.getParent() instanceof JComponent) && !(this.getView() instanceof JComponent))) {
            return false;
        }
        if (this.isPainting()) {
            return false;
        }
        Rectangle dirtyRegion = RepaintManager.currentManager(this).getDirtyRegion((JComponent)this.getParent());
        if (dirtyRegion != null && dirtyRegion.width > 0 && dirtyRegion.height > 0) {
            return false;
        }
        Rectangle rectangle = new Rectangle(0, 0, this.getWidth(), this.getHeight());
        Rectangle obj = new Rectangle();
        Rectangle bounds = null;
        Container container = null;
        while (this != null && JComponent.isLightweightComponent(this)) {
            int x = this.getX();
            int y = this.getY();
            int width = this.getWidth();
            int height = this.getHeight();
            obj.setBounds(rectangle);
            SwingUtilities.computeIntersection(0, 0, width, height, rectangle);
            if (!rectangle.equals(obj)) {
                return false;
            }
            if (container != null && this instanceof JComponent && !((JComponent)this).isOptimizedDrawingEnabled()) {
                Component[] components = this.getComponents();
                int i = 0;
                int j = components.length - 1;
                while (j >= 0) {
                    if (components[j] == container) {
                        i = j - 1;
                        break;
                    }
                    --j;
                }
                while (i >= 0) {
                    bounds = components[i].getBounds(bounds);
                    if (bounds.intersects(rectangle)) {
                        return false;
                    }
                    --i;
                }
            }
            rectangle.x += x;
            rectangle.y += y;
            container = this;
            this = this.getParent();
        }
        return this != null;
    }
    
    @Override
    public AccessibleContext getAccessibleContext() {
        if (this.accessibleContext == null) {
            this.accessibleContext = new JViewport.AccessibleJViewport();
        }
        return this.accessibleContext;
    }
    
    static {
        EnableWindowBlit = "EnableWindowBlit";
    }
    
    protected class ViewListener extends ComponentAdapter implements Serializable
    {
        @Override
        public void componentResized(ComponentEvent componentEvent) {
            JViewport.this.fireStateChanged();
            JViewport.this.revalidate();
        }
    }
    
    protected class AccessibleJViewport extends JComponent.AccessibleJComponent
    {
        @Override
        public AccessibleRole getAccessibleRole() {
            return AccessibleRole.VIEWPORT;
        }
    }
}
